<?php
namespace Bitrix\ImBot\Bot;

use Bitrix\Main\Localization\Loc;

Loc::loadMessages(__FILE__);

class TicketGate extends Base
{
	const URL = "https://my_ticket_place/rest.php"; // путь к файлу, который публикует информацию в тикеты
	const SECRET = "UNIQUE STRING"; // секретная строка для публикации сообщений

	const BOT_CODE = "TicketConnector";
	const INSTALL_WITH_MODULE = false;

	public static function register(array $params = Array())
	{
		if (!\Bitrix\Main\Loader::includeModule('im'))
			return false;

		$botId = \Bitrix\Im\Bot::register(Array(
			'CODE' => self::BOT_CODE,
			'TYPE' => \Bitrix\Im\Bot::TYPE_BOT,
			'MODULE_ID' => self::MODULE_ID,
			'CLASS' => __CLASS__,
			'LANG' => 'ru',
			'OPENLINE' => 'Y',
			'INSTALL_TYPE' => \Bitrix\Im\Bot::INSTALL_TYPE_SILENT,
			'METHOD_MESSAGE_ADD' => 'onMessageAdd',
			'METHOD_WELCOME_MESSAGE' => 'onChatStart',
			'METHOD_BOT_DELETE' => 'onBotDelete',
			'PROPERTIES' => Array(
				'NAME' => Loc::getMessage('IMBOT_BOT_NAME') ? Loc::getMessage('IMBOT_BOT_NAME') : self::BOT_CODE,
			)
		));
		if ($botId)
		{
			self::setBotId($botId);

			\Bitrix\Im\Command::register(Array(
						'MODULE_ID' => "imbot",
						'BOT_ID' => $botId,
						'HIDDEN' => 'Y',
						'COMMAND' => 'ticketCommand',
						'CLASS' => __CLASS__,
						'METHOD_COMMAND_ADD' => 'ticketCommand'
						));
		}

		return $botId;
	}

	public static function unRegister()
	{
		if (!\Bitrix\Main\Loader::includeModule('im'))
			return false;

		$result = \Bitrix\Im\Bot::unRegister(Array('BOT_ID' => self::getBotId()));
		if ($result)
			self::setBotId(0);

		return $result;
	}

	public static function onChatStart($dialogId, $joinFields)
	{
		$messageFields = $joinFields;
		$messageFields['DIALOG_ID'] = $dialogId;
		$messageFields['SYSTEM'] = 'Y';
		$messageFields['ANSWER'] = Loc::getMessage('IMBOT_BOT_ASK');
		
		$keyboard = new \Bitrix\Im\Bot\Keyboard(self::getBotId());
		$keyboard->addButton(Array(
					"TEXT" => Loc::getMessage('IMBOT_BOT_BTN_TICKET'),
					"COMMAND" => "ticketCommand",
					"COMMAND_PARAMS" => "ticket_new",
					"DISPLAY" => "LINE",
					"BLOCK" => "Y",
					));
		$keyboard->addButton(Array(
					"TEXT" => Loc::getMessage('IMBOT_BOT_BTN_ADMIN'),
					"COMMAND" => "ticketCommand",
					"COMMAND_PARAMS" => "delete_bot",
					"DISPLAY" => "LINE",
					"BLOCK" => "Y",
					));

		self::sendAnswer($messageFields, $keyboard);
		return true;
	}

	public static function onMessageAdd($messageId, $messageFields)
	{
		if ($messageFields['SYSTEM'] == 'Y')
			return false;

		if (!$messageFields['TO_CHAT_ID'])
		{
			$messageFields['SYSTEM'] = 'Y';
			$messageFields['ANSWER'] = Loc::getMessage('IMBOT_BOT_INTERNAL_ERROR');;
			self::sendAnswer($messageFields);
			return false;
		}

		if (defined('TICKET_TRACKBACK'))
			return true;

		if (!$f = self::getSession($messageFields))
		{
			$messageFields['SYSTEM'] = 'Y';
			$messageFields['ANSWER'] = Loc::getMessage('IMBOT_BOT_INTERNAL_ERROR');;
			self::sendAnswer($messageFields);
			return false;
		}

		$postFields = $messageFields;
		$postFields['NEXT_MESSAGE'] = 'Y';
		$postFields['AUTHOR_NAME'] = \Bitrix\Im\User::getInstance($messageFields['FROM_USER_ID'])->getFullName();
		$postFields['SESSION_ID'] = $f['ID'];
		$postFields['LANG'] = 'ru';

		$ar = self::ticketPost($postFields);
		if ($ar == false || $ar['status'] != 'success')
		{
			$messageFields['SYSTEM'] = 'Y';
			if ($message = $ar['message'])
				$messageFields['ANSWER'] = $message;
			else
				$messageFields['ANSWER'] = Loc::getMessage('IMBOT_BOT_INTERNAL_ERROR');

			self::sendAnswer($messageFields);
			
			$chat = new \Bitrix\ImOpenlines\Chat($messageFields['TO_CHAT_ID']);
			$chat->endBotSession();
		}
		return true;
	}

	public static function ticketCommand($messageId, $messageFields)
	{
		\CModule::IncludeModule('im');
		\CModule::IncludeModule('imopenlines');

		if (!$arSession = self::getSession($messageFields))
			return false;

		$arFilter = array(
			'CHAT_ID' => $arSession['CHAT_ID']
		);
		if ($arSession['START_ID'])
			$arFilter['>=ID'] = $arSession['START_ID'];
		if ($arSession['END_ID'])
			$arFilter['<=ID'] = $arSession['END_ID'];

		$postFields = $messageFields;
		$postFields['SESSION_ID'] = $arSession['ID'];
		$postFields['MESSAGES'] = array();
		$orm = \Bitrix\Im\Model\MessageTable::getList(array(
			'select' => array('ID', 'AUTHOR_ID', 'MESSAGE', 'DATE_CREATE'),
			'order' => array('ID' => 'ASC'),
			'filter' => $arFilter
		));
		while($f = $orm->fetch())
		{
			$f['DATE_CREATE'] = $f['DATE_CREATE']->toString();
			$f['AUTHOR_NAME'] = \Bitrix\Im\User::getInstance($f['AUTHOR_ID'])->getFullName();
			$postFields['MESSAGES'][] = $f;
		}
		
		if ($messageFields['COMMAND_PARAMS'] == 'ticket_new')
		{
			$ar = self::ticketPost($postFields);

			if ($ar !== false && $ar['status'] == 'success')
			{
				$keyboard = new \Bitrix\Im\Bot\Keyboard(self::getBotId());
				$keyboard->addButton(Array(
							"TEXT" => Loc::getMessage('IMBOT_BOT_BTN_REMOVE'),
							"COMMAND" => "ticketCommand",
							"COMMAND_PARAMS" => "disconnect",
							"DISPLAY" => "LINE",
							"BLOCK" => "Y",
							));
				\CIMMessageParam::Set($messageId, Array(
					'KEYBOARD' => 'N',
				));
				\CIMMessageParam::SendPull($messageId);
				$messageFields['SYSTEM'] = 'Y';
				$messageFields['ANSWER'] = $ar['message'];
				self::sendAnswer($messageFields, $keyboard);
			}
			else
			{
				$messageFields['SYSTEM'] = 'Y';
				if ($message = $ar['message'])
					$messageFields['ANSWER'] = $message;
				else
					$messageFields['ANSWER'] = Loc::getMessage('IMBOT_BOT_INTERNAL_ERROR');

				self::sendAnswer($messageFields);
				return false;
			}
		}
		elseif ($messageFields['COMMAND_PARAMS'] == 'disconnect')
		{
			$chat = new \Bitrix\ImOpenlines\Chat($messageFields['TO_CHAT_ID']);
			$chat->transfer(Array(
				'FROM' => $messageFields['FROM_USER_ID'],
				'SKIP_CHECK' => 'Y', // для передачи на бота
				'TO' => self::getBotId(),
				'MODE' => \Bitrix\ImOpenlines\Chat::TRANSFER_MODE_MANUAL,
				'LEAVE' => 'Y'
			));

			$keyboard = new \Bitrix\Im\Bot\Keyboard(self::getBotId());
			$keyboard->addButton(Array(
						"TEXT" => Loc::getMessage('IMBOT_BOT_BTN_ADMIN'),
						"COMMAND" => "ticketCommand",
						"COMMAND_PARAMS" => "delete_bot",
						"DISPLAY" => "LINE",
						"BLOCK" => "Y",
						));
			\CIMMessageParam::Set($messageId, Array('KEYBOARD' => $keyboard));
			\CIMMessageParam::SendPull($messageId);
		}
		elseif ($messageFields['COMMAND_PARAMS'] == 'delete_bot')
		{
			$chat = new \Bitrix\ImOpenlines\Chat($messageFields['TO_CHAT_ID']);
			$chat->transfer(Array(
				'FROM' => self::getBotId(),
				'TO' => $messageFields['FROM_USER_ID'],
				'MODE' => \Bitrix\ImOpenlines\Chat::TRANSFER_MODE_MANUAL,
				'LEAVE' => 'Y'
			));
			\CIMMessageParam::Set($messageId, Array(
				'KEYBOARD' => 'N',
			));
			\CIMMessageParam::SendPull($messageId);
		}

		return true;
	}

	public static function sendAnswer($messageFields, $keyboard = array())
	{
		\Bitrix\Im\Bot::addMessage(Array('BOT_ID' => self::getBotId()), Array(
			'DIALOG_ID' => $messageFields['DIALOG_ID'],
			'MESSAGE' => $messageFields['ANSWER'],
			'SYSTEM' => $messageFields['SYSTEM'] == 'Y' ? 'Y' : 'N',
			'KEYBOARD' => $keyboard,
			'URL_PREVIEW' => isset($messageFields['RICH'])? $messageFields['RICH']: "Y"
		));
	}

	public static function ticketPost($params)
	{
		global $APPLICATION;
		$httpClient = new \Bitrix\Main\Web\HttpClient(array(
					"socketTimeout" => 20,
					"streamTimeout" => 60,
					"disableSslVerification" => true
				));
		$httpClient->setHeader('User-Agent', 'Bitrix Ticket Connector');
		$params['action'] = 'TicketBot';
		$params['timestamp'] = time();
		$params['hash'] = md5($params['timestamp'].self::SECRET);

		$url = self::URL;
		$result = $httpClient->post($url, $params);
		try
		{
			$ar = \Bitrix\Main\Web\Json::decode($result);
		}
		catch (\Bitrix\Main\ArgumentException $e)
		{
			AddMessage2Log($result);
			echo $result;
			return false;
		}
		return $ar;
	}

	public static function getSession($messageFields)
	{
		\CModule::IncludeModule('im');
		\CModule::IncludeModule('imopenlines');

		return \Bitrix\ImOpenlines\Model\SessionTable::getList(Array(
			'select' => Array('ID', 'CHAT_ID', 'START_ID', 'END_ID', 'CONFIG_ID'),
			'filter' => Array(
				'=USER_CODE' => $messageFields['CHAT_ENTITY_ID'],
				'=CLOSED' => 'N'
			)
		))->fetch();
	}
}
